"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const types_1 = require("./types");
const useCustomOptions_1 = __importDefault(require("./useCustomOptions"));
const useRequestInit_1 = __importDefault(require("./useRequestInit"));
const use_ssr_1 = __importDefault(require("use-ssr"));
const makeRouteAndOptions_1 = __importDefault(require("./makeRouteAndOptions"));
// No <Provider url='example.com' />
// function useFetch<TData = any>(url: string, options?: NoUrlOptions): UseFetch<TData>
// function useFetch<TData = any>(options: Options): UseFetch<TData>
// With <Provider url='example.com' />
// options should be extended. In future maybe have options callback to completely overwrite options
// i.e. useFetch('ex.com', oldOptions => ({ ...newOptions })) to overwrite
// function useFetch<TData = any>(url?: string, options?: NoUrlOptions): UseFetch<TData>
// function useFetch<TData = any>(options?: OptionsMaybeURL): UseFetch<TData>
function useFetch(...args) {
    const _a = useCustomOptions_1.default(...args), { url, onMount, path } = _a, defaults = __rest(_a, ["url", "onMount", "path"]);
    const requestInit = useRequestInit_1.default(...args);
    const { isBrowser, isServer } = use_ssr_1.default();
    const controller = react_1.useRef();
    const res = react_1.useRef();
    const data = react_1.useRef(defaults.data);
    const [loading, setLoading] = react_1.useState(defaults.loading);
    const [error, setError] = react_1.useState();
    const makeFetch = react_1.useCallback((method) => {
        return (routeOrBody, body) => __awaiter(this, void 0, void 0, function* () {
            controller.current = isBrowser ? new AbortController() : null;
            const { route, options } = makeRouteAndOptions_1.default(requestInit, method, controller, routeOrBody, body);
            try {
                setLoading(true);
                if (isServer)
                    return; // TODO: for now, we don't do anything on the server
                res.current = yield fetch(`${url}${path}${route}`, options);
                try {
                    data.current = yield res.current.json();
                }
                catch (err) {
                    data.current = (yield res.current.text()); // FIXME: should not be `any` type
                }
            }
            catch (err) {
                if (err.name !== 'AbortError')
                    setError(err);
            }
            finally {
                controller.current = null;
                setLoading(false);
            }
            return data.current;
        });
    }, [url, isBrowser, requestInit, isServer]);
    const post = makeFetch(types_1.HTTPMethod.POST);
    const del = makeFetch(types_1.HTTPMethod.DELETE);
    const request = {
        get: makeFetch(types_1.HTTPMethod.GET),
        post,
        patch: makeFetch(types_1.HTTPMethod.PATCH),
        put: makeFetch(types_1.HTTPMethod.PUT),
        del,
        delete: del,
        abort() {
            controller.current && controller.current.abort();
        },
        query: (query, variables) => post({ query, variables }),
        mutate: (mutation, variables) => post({ mutation, variables }),
        loading: loading,
        error,
        data: data.current,
    };
    const response = Object.assign({ data: data.current }, res.current);
    // handling onMount
    const mounted = react_1.useRef(false);
    react_1.useEffect(() => {
        if (!onMount || mounted.current)
            return;
        mounted.current = true;
        const methodName = requestInit.method || types_1.HTTPMethod.GET;
        const methodLower = methodName.toLowerCase();
        if (methodName !== types_1.HTTPMethod.GET) {
            const req = request[methodLower];
            req(requestInit.body);
        }
        else {
            const req = request[methodLower];
            req();
        }
    }, [onMount, requestInit.body, requestInit.method, url]);
    return Object.assign([request, response, loading, error], Object.assign({ request, response: response }, request));
}
exports.useFetch = useFetch;
exports.default = useFetch;
//# sourceMappingURL=useFetch.js.map